@@ -1,31 +1,33 @@
-import abc
 import atexit
 import os
 import math
 import typing
+from typing import Union
 from rpy2.rinterface_lib import openrlib
 import rpy2.rinterface_lib._rinterface_capi as _rinterface
 import rpy2.rinterface_lib.embedded as embedded
 import rpy2.rinterface_lib.conversion as conversion
+from rpy2.rinterface_lib.conversion import _cdata_res_to_rinterface
 import rpy2.rinterface_lib.memorymanagement as memorymanagement
-import rpy2.rinterface_lib.na_values as na_values
+from rpy2.rinterface_lib import na_values
+from rpy2.rinterface_lib.sexp import NULLType
 import rpy2.rinterface_lib.bufferprotocol as bufferprotocol
-import rpy2.rinterface_lib.sexp as sexp
+from rpy2.rinterface_lib import sexp
+from rpy2.rinterface_lib.sexp import CharSexp  # noqa: F401
+from rpy2.rinterface_lib.sexp import RTYPES
+from rpy2.rinterface_lib.sexp import SexpVector
+from rpy2.rinterface_lib.sexp import StrSexpVector
+from rpy2.rinterface_lib.sexp import Sexp
+from rpy2.rinterface_lib.sexp import SexpEnvironment
+from rpy2.rinterface_lib.sexp import unserialize  # noqa: F401
+from rpy2.rinterface_lib.sexp import emptyenv
+from rpy2.rinterface_lib.sexp import baseenv
+from rpy2.rinterface_lib.sexp import globalenv
 
 if os.name == 'nt':
     import rpy2.rinterface_lib.embedded_mswin as embedded_mswin
-    embedded._initr = embedded_mswin._initr
+    embedded._initr = embedded_mswin._initr_win32
 
-Sexp = sexp.Sexp
-StrSexpVector = sexp.StrSexpVector
-CharSexp = sexp.CharSexp
-SexpVector = sexp.SexpVector
-RTYPES = sexp.RTYPES
-
-unserialize = sexp.unserialize
-
-_cdata_res_to_rinterface = conversion._cdata_res_to_rinterface
-_evaluated_promise = _rinterface._evaluated_promise
 R_NilValue = openrlib.rlib.R_NilValue
 
 endr = embedded.endr
@@ -43,7 +45,9 @@ def parse(text: str, num: int = -1):
     if not isinstance(text, str):
         raise TypeError('text must be a string.')
     robj = StrSexpVector([text])
-    return _rinterface._parse(robj.__sexp__._cdata, num)
+    with memorymanagement.rmemory() as rmemory:
+        res = _rinterface._parse(robj.__sexp__._cdata, num, rmemory)
+    return res
 
 
 def evalr(source: str, maxlines: int = -1) -> sexp.Sexp:
@@ -89,58 +93,17 @@ def vector_memoryview(obj: sexp.SexpVector,
     return mv
 
 
-class NULLType(sexp.Sexp, metaclass=na_values.Singleton):
-    """A singleton class for R's NULL."""
-
-    def __init__(self):
-        embedded.assert_isready()
-        super().__init__(
-            sexp.Sexp(
-                _rinterface.UnmanagedSexpCapsule(
-                    openrlib.rlib.R_NilValue
-                )
-            )
-        )
-
-    def __bool__(self) -> bool:
-        """This is always False."""
-        return False
-
-    @property
-    def __sexp__(self) -> _rinterface.SexpCapsule:
-        return self._sexpobject
-
-    @property
-    def rid(self) -> int:
-        return self._sexpobject.rid
-
-
-class _MissingArgType(sexp.Sexp, metaclass=na_values.Singleton):
-
-    def __init__(self):
-        embedded.assert_isready()
-        super().__init__(
-            sexp.Sexp(
-                _rinterface.UnmanagedSexpCapsule(
-                    openrlib.rlib.R_MissingArg
-                )
-            )
-        )
-
-    def __bool__(self) -> bool:
-        """This is always False."""
-        return False
-
-    @property
-    def __sexp__(self) -> _rinterface.SexpCapsule:
-        return self._sexpobject
-
-
 class SexpSymbol(sexp.Sexp):
     """An unevaluated R symbol."""
 
-    def __init__(self, obj):
-        if isinstance(obj, Sexp) or isinstance(obj, _rinterface.SexpCapsule):
+    def __init__(
+            self,
+            obj: Union[Sexp,
+                       _rinterface.SexpCapsule,
+                       _rinterface.UninitializedRCapsule,
+                       str]
+    ):
+        if isinstance(obj, Sexp) or isinstance(obj, _rinterface.CapsuleBase):
             super().__init__(obj)
         elif isinstance(obj, str):
             name_cdata = _rinterface.ffi.new('char []', obj.encode('utf-8'))
@@ -161,176 +124,31 @@ class SexpSymbol(sexp.Sexp):
         )
 
 
-class SexpEnvironment(sexp.Sexp):
-    """Proxy for an R "environment" object.
+class _MissingArgType(SexpSymbol, metaclass=sexp.SingletonABC):
 
-    An R "environment" object can be thought of as a mix of a
-    mapping (like a `dict`) and a scope. To make it more "Pythonic",
-    both aspects are kept separate and the method `__getitem__` will
-    get an item as it would for a Python `dict` while the method `find`
-    will get an item as if it was a scope.
-
-    As soon as R is initialized the following main environments become
-    available to the user:
-    - `globalenv`: The "workspace" for the current R process. This can
-      be thought of as when `__name__ == '__main__'` in Python.
-    - `baseenv`: The namespace of R's "base" package.
-    """
-
-    @_cdata_res_to_rinterface
-    @_evaluated_promise
-    def find(self,
-             key: str,
-             wantfun: bool = False) -> sexp.Sexp:
-        """Find an item, starting with this R environment.
-
-        Raises a `KeyError` if the key cannot be found.
-
-        This method is called `find` because it is somewhat different
-        from the method :meth:`get` in Python mappings such :class:`dict`.
-        This is looking for a key across enclosing environments, returning
-        the first key found."""
-
-        if not isinstance(key, str):
-            raise TypeError('The key must be a non-empty string.')
-        elif not len(key):
-            raise ValueError('The key must be a non-empty string.')
-        with memorymanagement.rmemory() as rmemory:
-            symbol = rmemory.protect(
-                openrlib.rlib.Rf_install(conversion._str_to_cchar(key))
+    def __init__(self):
+        if embedded.isready():
+            tmp = sexp.Sexp(
+                _rinterface.UnmanagedSexpCapsule(
+                    openrlib.rlib.R_MissingArg
+                )
             )
-            if wantfun:
-                # One would expect this to be like
-                #   res = _rinterface._findfun(symbol, self.__sexp__._cdata)
-                # but R's findfun will segfault if the symbol is not in
-                # the environment. :/
-                rho = self
-                while rho.rid != emptyenv.rid:
-                    res = _rinterface._findVarInFrame(symbol,
-                                                      rho.__sexp__._cdata)
-                    if _rinterface._TYPEOF(res) in (openrlib.rlib.CLOSXP,
-                                                    openrlib.rlib.BUILTINSXP):
-                        break
-                    # TODO: move check of R_UnboundValue to _rinterface ?
-                    res = openrlib.rlib.R_UnboundValue
-                    rho = rho.enclos
-            else:
-                res = _rinterface._findvar(symbol, self.__sexp__._cdata)
-        # TODO: move check of R_UnboundValue to _rinterface ?
-        if res == openrlib.rlib.R_UnboundValue:
-            raise KeyError("'%s' not found" % key)
-        return res
-
-    @_cdata_res_to_rinterface
-    @_evaluated_promise
-    def __getitem__(self, key: str) -> typing.Any:
-        if not isinstance(key, str):
-            raise TypeError('The key must be a non-empty string.')
-        elif not len(key):
-            raise ValueError('The key must be a non-empty string.')
-        with memorymanagement.rmemory() as rmemory:
-            symbol = rmemory.protect(
-                openrlib.rlib.Rf_install(conversion._str_to_cchar(key))
+        else:
+            tmp = sexp.Sexp(
+                _rinterface.UninitializedRCapsule(RTYPES.SYMSXP.value)
             )
-            res = _rinterface._findVarInFrame(symbol, self.__sexp__._cdata)
-        # TODO: move check of R_UnboundValue to _rinterface
-        if res == openrlib.rlib.R_UnboundValue:
-            raise KeyError("'%s' not found" % key)
-        return res
+        super().__init__(tmp)
 
-    def __setitem__(self, key: str, value) -> None:
-        # TODO: move body to _rinterface-level function
-        if not isinstance(key, str):
-            raise TypeError('The key must be a non-empty string.')
-        elif not len(key):
-            raise ValueError('The key must be a non-empty string.')
-        if (self.__sexp__._cdata == openrlib.rlib.R_BaseEnv) or \
-           (self.__sexp__._cdata == openrlib.rlib.R_EmptyEnv):
-            raise ValueError('Cannot remove variables from the base or '
-                             'empty environments.')
-        # TODO: call to Rf_duplicate needed ?
-        with memorymanagement.rmemory() as rmemory:
-            symbol = rmemory.protect(
-                openrlib.rlib.Rf_install(conversion._str_to_cchar(key))
-            )
-            cdata = rmemory.protect(conversion._get_cdata(value))
-            cdata_copy = rmemory.protect(
-                openrlib.rlib.Rf_duplicate(cdata)
-            )
-            openrlib.rlib.Rf_defineVar(symbol,
-                                       cdata_copy,
-                                       self.__sexp__._cdata)
-
-    def __len__(self) -> int:
-        with memorymanagement.rmemory() as rmemory:
-            symbols = rmemory.protect(
-                openrlib.rlib.R_lsInternal(self.__sexp__._cdata,
-                                           openrlib.rlib.TRUE)
-            )
-            n = openrlib.rlib.Rf_xlength(symbols)
-        return n
-
-    def __delitem__(self, key: str) -> None:
-        # Testing that key is a non-empty string is implicitly
-        # performed when checking that the key is in the environment.
-        if key not in self:
-            raise KeyError("'%s' not found" % key)
-
-        if self.__sexp__ == baseenv.__sexp__:
-            raise ValueError('Values from the R base environment '
-                             'cannot be removed.')
-        # TODO: also check it is not R_EmpyEnv or R_BaseNamespace
-        if self.is_locked():
-            ValueError('Cannot remove an item from a locked '
-                       'environment.')
-
-        with memorymanagement.rmemory() as rmemory:
-            key_cdata = rmemory.protect(
-                openrlib.rlib.Rf_mkString(conversion._str_to_cchar(key))
-            )
-            _rinterface._remove(key_cdata,
-                                self.__sexp__._cdata,
-                                openrlib.rlib.Rf_ScalarLogical(
-                                    openrlib.rlib.FALSE))
-
-    @_cdata_res_to_rinterface
-    def frame(self) -> 'typing.Union[NULLType, SexpEnvironment]':
-        """Get the parent frame of the environment."""
-        return openrlib.rlib.FRAME(self.__sexp__._cdata)
+    def __bool__(self) -> bool:
+        """This is always False."""
+        return False
 
     @property
-    @_cdata_res_to_rinterface
-    def enclos(self) -> 'typing.Union[NULLType, SexpEnvironment]':
-        """Get or set the enclosing environment."""
-        return openrlib.rlib.ENCLOS(self.__sexp__._cdata)
+    def __sexp__(self) -> _rinterface.SexpCapsule:
+        return self._sexpobject
 
-    @enclos.setter
-    def enclos(self, value: 'SexpEnvironment') -> None:
-        assert isinstance(value, SexpEnvironment)
-        openrlib.rlib.SET_ENCLOS(self.__sexp__._cdata,
-                                 value.__sexp__.cdata)
 
-    def keys(self) -> typing.Generator[str, None, None]:
-        """Generator over the keys (symbols) in the environment."""
-        with memorymanagement.rmemory() as rmemory:
-            symbols = rmemory.protect(
-                openrlib.rlib.R_lsInternal(self.__sexp__._cdata,
-                                           openrlib.rlib.TRUE)
-            )
-            n = openrlib.rlib.Rf_xlength(symbols)
-            res = []
-            for i in range(n):
-                res.append(_rinterface._string_getitem(symbols, i))
-        for e in res:
-            yield e
-
-    def __iter__(self) -> typing.Generator[str, None, None]:
-        """See method `keys()`."""
-        return self.keys()
-
-    def is_locked(self) -> bool:
-        return openrlib.rlib.R_EnvironmentIsLocked(
-            self.__sexp__._cdata)
+MissingArg = _MissingArgType()
 
 
 class SexpPromise(Sexp):
@@ -343,18 +161,27 @@ class SexpPromise(Sexp):
           promise.
         """
         if not env:
-            env = embedded.globalenv
+            env = globalenv
         return openrlib.rlib.Rf_eval(self.__sexp__._cdata, env)
 
 
-class NumpyArrayInterface(abc.ABC):
+NPCOMPAT_TYPE = typing.TypeVar('NPCOMPAT_TYPE',
+                               'ByteSexpVector',
+                               'BoolSexpVector',
+                               'IntSexpVector',
+                               'FloatSexpVector')
+
+
+class NumpyArrayMixin:
     """Numpy-specific API for accessing the content of a numpy array.
 
     This interface implements version 3 of Numpy's `__array_interface__`
     and is only available / possible for some of the R vectors."""
 
     @property
-    def __array_interface__(self) -> dict:
+    def __array_interface__(
+            self: NPCOMPAT_TYPE
+    ) -> dict:
         """Return an `__array_interface__` version 3.
 
         Note that the pointer returned in the items 'data' corresponds to
@@ -374,7 +201,7 @@ class NumpyArrayInterface(abc.ABC):
                 'version': 3}
 
 
-class ByteSexpVector(SexpVector, NumpyArrayInterface):
+class ByteSexpVector(NumpyArrayMixin, SexpVector):
     """Array of bytes.
 
     This is the R equivalent to a Python :class:`bytesarray`.
@@ -409,7 +236,9 @@ class ByteSexpVector(SexpVector, NumpyArrayInterface):
     def _R_SET_VECTOR_ELT(x, i: int, val) -> None:
         openrlib.RAW(x)[i] = val
 
-    def __getitem__(self, i: int) -> typing.Union[int, 'ByteSexpVector']:
+    def __getitem__(self,
+                    i: Union[int, slice]) -> Union[int, 'ByteSexpVector']:
+
         cdata = self.__sexp__._cdata
         if isinstance(i, int):
             i_c = _rinterface._python_index_to_c(cdata, i)
@@ -426,7 +255,7 @@ class ByteSexpVector(SexpVector, NumpyArrayInterface):
                 'Indices must be integers or slices, not %s' % type(i))
         return res
 
-    def __setitem__(self, i: int, value) -> None:
+    def __setitem__(self, i: Union[int, slice], value) -> None:
         cdata = self.__sexp__._cdata
         if isinstance(i, int):
             i_c = _rinterface._python_index_to_c(cdata, i)
@@ -441,7 +270,7 @@ class ByteSexpVector(SexpVector, NumpyArrayInterface):
                 'Indices must be integers or slices, not %s' % type(i))
 
 
-class BoolSexpVector(SexpVector, NumpyArrayInterface):
+class BoolSexpVector(NumpyArrayMixin, SexpVector):
     """Array of booleans.
 
     Note that R is internally storing booleans as integers to
@@ -461,8 +290,8 @@ class BoolSexpVector(SexpVector, NumpyArrayInterface):
         else:
             return bool(x)
 
-    def __getitem__(self, i: int) -> typing.Union[typing.Optional[bool],
-                                                  'BoolSexpVector']:
+    def __getitem__(self, i: Union[int, slice]) -> Union[typing.Optional[bool],
+                                                         'BoolSexpVector']:
         cdata = self.__sexp__._cdata
         if isinstance(i, int):
             i_c = _rinterface._python_index_to_c(cdata, i)
@@ -478,7 +307,7 @@ class BoolSexpVector(SexpVector, NumpyArrayInterface):
                 'Indices must be integers or slices, not %s' % type(i))
         return res
 
-    def __setitem__(self, i: int, value) -> None:
+    def __setitem__(self, i: Union[int, slice], value) -> None:
         cdata = self.__sexp__._cdata
         if isinstance(i, int):
             i_c = _rinterface._python_index_to_c(cdata, i)
@@ -503,7 +332,7 @@ def nullable_int(v):
         return int(v)
 
 
-class IntSexpVector(SexpVector, NumpyArrayInterface):
+class IntSexpVector(NumpyArrayMixin, SexpVector):
 
     _R_TYPE = openrlib.rlib.INTSXP
     _R_SET_VECTOR_ELT = openrlib.SET_INTEGER_ELT
@@ -514,7 +343,7 @@ class IntSexpVector(SexpVector, NumpyArrayInterface):
     _R_GET_PTR = staticmethod(openrlib.INTEGER)
     _CAST_IN = staticmethod(nullable_int)
 
-    def __getitem__(self, i: int) -> typing.Union[int, 'IntSexpVector']:
+    def __getitem__(self, i: Union[int, slice]) -> Union[int, 'IntSexpVector']:
         cdata = self.__sexp__._cdata
         if isinstance(i, int):
             i_c = _rinterface._python_index_to_c(cdata, i)
@@ -532,7 +361,7 @@ class IntSexpVector(SexpVector, NumpyArrayInterface):
                 'Indices must be integers or slices, not %s' % type(i))
         return res
 
-    def __setitem__(self, i: int, value) -> None:
+    def __setitem__(self, i: Union[int, slice], value) -> None:
         cdata = self.__sexp__._cdata
         if isinstance(i, int):
             i_c = _rinterface._python_index_to_c(cdata, i)
@@ -550,7 +379,7 @@ class IntSexpVector(SexpVector, NumpyArrayInterface):
         return vector_memoryview(self, 'int', 'i')
 
 
-class FloatSexpVector(SexpVector, NumpyArrayInterface):
+class FloatSexpVector(NumpyArrayMixin, SexpVector):
 
     _R_TYPE = openrlib.rlib.REALSXP
     _R_VECTOR_ELT = openrlib.REAL_ELT
@@ -561,7 +390,9 @@ class FloatSexpVector(SexpVector, NumpyArrayInterface):
     _CAST_IN = staticmethod(float)
     _R_GET_PTR = staticmethod(openrlib.REAL)
 
-    def __getitem__(self, i: int) -> typing.Union[float, 'FloatSexpVector']:
+    def __getitem__(
+            self, i: Union[int, slice]
+    ) -> Union[float, 'FloatSexpVector']:
         cdata = self.__sexp__._cdata
         if isinstance(i, int):
             i_c = _rinterface._python_index_to_c(cdata, i)
@@ -576,7 +407,7 @@ class FloatSexpVector(SexpVector, NumpyArrayInterface):
                             type(i))
         return res
 
-    def __setitem__(self, i: int, value) -> None:
+    def __setitem__(self, i: Union[int, slice], value) -> None:
         cdata = self.__sexp__._cdata
         if isinstance(i, int):
             i_c = _rinterface._python_index_to_c(cdata, i)
@@ -621,8 +452,9 @@ class ComplexSexpVector(SexpVector):
                 )
         return res
 
-    def __getitem__(self,
-                    i: int) -> typing.Union[complex, 'ComplexSexpVector']:
+    def __getitem__(
+            self, i: Union[int, slice]
+    ) -> Union[complex, 'ComplexSexpVector']:
         cdata = self.__sexp__._cdata
         if isinstance(i, int):
             i_c = _rinterface._python_index_to_c(cdata, i)
@@ -638,7 +470,7 @@ class ComplexSexpVector(SexpVector):
                 'Indices must be integers or slices, not %s' % type(i))
         return res
 
-    def __setitem__(self, i: int, value) -> None:
+    def __setitem__(self, i: Union[int, slice], value) -> None:
         cdata = self.__sexp__._cdata
         if isinstance(i, int):
             i_c = _rinterface._python_index_to_c(cdata, i)
@@ -680,7 +512,7 @@ class PairlistSexpVector(SexpVector):
     _R_SET_VECTOR_ELT = None
     _CAST_IN = staticmethod(conversion._get_cdata)
 
-    def __getitem__(self, i: int) -> Sexp:
+    def __getitem__(self, i: Union[int, slice]) -> Sexp:
         cdata = self.__sexp__._cdata
         rlib = openrlib.rlib
         if isinstance(i, int):
@@ -762,7 +594,7 @@ class LangSexpVector(SexpVector):
             openrlib.rlib.Rf_nthcdr(cdata, i_c)
         )
 
-    def __setitem__(self, i: int, value) -> None:
+    def __setitem__(self, i: int, value: sexp.SupportsSEXP) -> None:
         cdata = self.__sexp__._cdata
         i_c = _rinterface._python_index_to_c(cdata, i)
         openrlib.rlib.SETCAR(
@@ -783,7 +615,7 @@ class SexpClosure(Sexp):
             res = rmemory.protect(
                 openrlib.rlib.R_tryEval(
                     call_r,
-                    embedded.globalenv.__sexp__._cdata,
+                    globalenv.__sexp__._cdata,
                     error_occured))
             if error_occured[0]:
                 raise embedded.RRuntimeError(_rinterface._geterrmessage())
@@ -906,20 +738,20 @@ conversion._PY_R_MAP.update({
     _rinterface.ffi.CData: False,
     # integer
     int: conversion._int_to_sexp,
-    na_values.NAIntegerType: conversion._int_to_sexp,
+    sexp.NAIntegerType: conversion._int_to_sexp,
     # float
     float: conversion._float_to_sexp,
-    na_values.NARealType: conversion._float_to_sexp,
+    sexp.NARealType: conversion._float_to_sexp,
     # boolean
     bool: conversion._bool_to_sexp,
-    na_values.NALogicalType: conversion._bool_to_sexp,
+    sexp.NALogicalType: conversion._bool_to_sexp,
     # string
     str: conversion._str_to_sexp,
     sexp.CharSexp: None,
-    na_values.NACharacterType: None,
+    sexp.NACharacterType: None,
     # complex
     complex: conversion._complex_to_sexp,
-    na_values.NAComplexType: conversion._complex_to_sexp,
+    sexp.NAComplexType: conversion._complex_to_sexp,
     # None
     type(None): lambda x: openrlib.rlib.R_NilValue})
 
@@ -948,11 +780,8 @@ class RRuntimeWarning(RuntimeWarning):
     pass
 
 
-emptyenv = None
-baseenv = None
-globalenv = None
 NULL = None
-MissingArg = None
+MissingArg = _MissingArgType()
 NA_Character = None
 NA_Integer = None
 NA_Logical = None
@@ -961,7 +790,7 @@ NA_Real = None
 NA_Complex = None
 
 
-def initr_simple() -> int:
+def initr_simple() -> typing.Optional[int]:
     """Initialize R's embedded C library."""
     with openrlib.rlock:
         status = embedded._initr()
@@ -971,7 +800,7 @@ def initr_simple() -> int:
         return status
 
 
-def initr_checkenv() -> typing.Union[int, None]:
+def initr_checkenv() -> typing.Optional[int]:
     # Force the internal initialization flag if there is an environment
     # variable that indicates that R was alreay initialized in the current
     # process.
@@ -993,49 +822,36 @@ initr = initr_checkenv
 
 def _post_initr_setup() -> None:
 
-    embedded.emptyenv = SexpEnvironment(
-        _rinterface.SexpCapsule(openrlib.rlib.R_EmptyEnv)
-    )
-    global emptyenv
-    emptyenv = embedded.emptyenv
-
-    embedded.baseenv = SexpEnvironment(
-        _rinterface.SexpCapsule(openrlib.rlib.R_BaseEnv)
-    )
-    global baseenv
-    baseenv = embedded.baseenv
-
-    embedded.globalenv = SexpEnvironment(
-        _rinterface.SexpCapsule(openrlib.rlib.R_GlobalEnv)
-    )
-    global globalenv
-    globalenv = embedded.globalenv
+    emptyenv.__sexp__ = _rinterface.SexpCapsule(openrlib.rlib.R_EmptyEnv)
+    baseenv.__sexp__ = _rinterface.SexpCapsule(openrlib.rlib.R_BaseEnv)
+    globalenv.__sexp__ = _rinterface.SexpCapsule(openrlib.rlib.R_GlobalEnv)
 
     global NULL
     NULL = NULLType()
 
-    global MissingArg
-    MissingArg = _MissingArgType()
+    MissingArg._sexpobject = _rinterface.UnmanagedSexpCapsule(
+        openrlib.rlib.R_MissingArg
+    )
 
     global NA_Character
-    na_values.NA_Character = na_values.NACharacterType()
+    na_values.NA_Character = sexp.NACharacterType()
     NA_Character = na_values.NA_Character
 
     global NA_Integer
-    na_values.NA_Integer = na_values.NAIntegerType(openrlib.rlib.R_NaInt)
+    na_values.NA_Integer = sexp.NAIntegerType(openrlib.rlib.R_NaInt)
     NA_Integer = na_values.NA_Integer
 
     global NA_Logical, NA
-    na_values.NA_Logical = na_values.NALogicalType(openrlib.rlib.R_NaInt)
+    na_values.NA_Logical = sexp.NALogicalType(openrlib.rlib.R_NaInt)
     NA_Logical = na_values.NA_Logical
     NA = NA_Logical
 
     global NA_Real
-    na_values.NA_Real = na_values.NARealType(openrlib.rlib.R_NaReal)
+    na_values.NA_Real = sexp.NARealType(openrlib.rlib.R_NaReal)
     NA_Real = na_values.NA_Real
 
     global NA_Complex
-    na_values.NA_Complex = na_values.NAComplexType(
+    na_values.NA_Complex = sexp.NAComplexType(
         _rinterface.ffi.new(
             'Rcomplex *',
             [openrlib.rlib.R_NaReal, openrlib.rlib.R_NaReal])
