@@ -69,12 +69,13 @@ def r_home_from_subprocess() -> Optional[str]:
     return res
 
 
+# TODO: move all Windows all code into an os-specific module ?
 def r_home_from_registry() -> Optional[str]:
     """Return the R home directory from the Windows Registry."""
     try:
-        import winreg
+        import winreg  # type: ignore
     except ImportError:
-        import _winreg as winreg
+        import _winreg as winreg  # type: ignore
     try:
         hkey = winreg.OpenKeyEx(winreg.HKEY_LOCAL_MACHINE,
                                 'Software\\R-core\\R',
@@ -88,12 +89,40 @@ def r_home_from_registry() -> Optional[str]:
     return r_home
 
 
+def r_ld_library_path_from_subprocess(r_home: str) -> str:
+    """Get the LD_LIBRARY_PATH settings added by R."""
+    cmd = (os.path.join(r_home, 'bin', 'Rscript'),
+           '-e',
+           'cat(Sys.getenv("LD_LIBRARY_PATH"))')
+    try:
+        r_lib_path = subprocess.check_output(cmd, universal_newlines=True)
+    except Exception:  # FileNotFoundError, WindowsError, etc
+        r_lib_path = ''
+    res = None
+    ld_library_path = os.environ.get('LD_LIBRARY_PATH')
+    if ld_library_path:
+        pos = r_lib_path.find(ld_library_path)
+        if pos != -1:
+            res = (r_lib_path[pos:(pos+len(ld_library_path))]
+                   .rstrip(os.pathsep))
+    if res is None:
+        res = r_lib_path
+    return res
+
+
 def get_rlib_path(r_home: str, system: str) -> str:
     """Get the path for the R shared library."""
     if system == 'Linux':
         lib_path = os.path.join(r_home, 'lib', 'libR.so')
     elif system == 'Darwin':
         lib_path = os.path.join(r_home, 'lib', 'libR.dylib')
+    elif system == 'Windows':
+        # i386
+        os.environ['PATH'] = ''.join(
+            (os.environ['PATH'],
+             os.path.join(r_home, 'bin', 'x64'))
+        )
+        lib_path = os.path.join(r_home, 'bin', 'x64', 'R.dll')
     else:
         raise ValueError(
             'The system {system} is currently not supported.'
@@ -263,15 +292,21 @@ def iter_info():
         if r_home_default is None:
             yield ('    Warning: There is no R in the PATH and no '
                    'R_HOME defined.')
+            return
         else:
             r_home = r_home_default
 
+    yield _make_bold("R's additions to LD_LIBRARY_PATH:")
+    yield r_ld_library_path_from_subprocess(r_home)
+
     if has_rpy2:
         try:
             import rpy2.rinterface_lib.openrlib
             rlib_status = 'OK'
         except ImportError as ie:
             rlib_status = '*** Error while loading: %s ***' % str(ie)
+        except OSError as ose:
+            rlib_status = str(ose)
     else:
         rlib_status = '*** rpy2 is not installed'
 
@@ -308,6 +343,20 @@ if __name__ == '__main__':
     parser = argparse.ArgumentParser(
         'Command-line tool to report the rpy2'
         'environment and help diagnose issues')
+    parser.add_argument('action',
+                        nargs='?',
+                        choices=('info', 'LD_LIBRARY_PATH'),
+                        default='info',
+                        help=('Action to perform. "info" shows all info, '
+                              'LD_LIBRARY_PATH returns optionally required '
+                              'additions to the environment variable'))
     args = parser.parse_args()
-    for row in iter_info():
-        print(row)
+    if args.action == 'info':
+        for row in iter_info():
+            print(row)
+    elif args.action == 'LD_LIBRARY_PATH':
+        r_home = get_r_home()
+        if not r_home:
+            print('R cannot be found in the PATH and RHOME cannot be found.')
+            sys.exit(1)
+        print(r_ld_library_path_from_subprocess(r_home))
